#!/usr/bin/perl

# Marc Quinton / juil 2000 / http://quinton.free.fr/MagickThumbnail/
#
# create thumbnail, html index page, optionaly resample images
# to minimise file size
#
# use ImageMagick::convert external program file
# especialy -geometry, -sample, and -quality options
# see man convert, rpm -qi ImageMagick
#
# usage : mthumbnail -h -v -r -d -C -R -f -c num -q num -g geo -s geo [<dir>]+
# see mthumbnail -h for all options.
#
# MODIFIED BY hossman@fucit.org ON 2001-05-28.  ADDED RESIZING SUPPORT
# FOR SOURCE IMAGE, CHANGED page.html TO index.html, AND CHANGED THE
# FORMAT OF THE RESULTING HTML.  ALSO FIXED '-H' OPTION SO IT WORKS
# EVEN IF THE POD FILE ISN'T INSTALLED.  ALSO DISPLAYS THE FILES IN
# THE ORDER THE APPEAR IN THE DESCRIPTION FILE (IF IT EXISTS) .. IF
# SOME FILES AREN'T IN THE DESCRIPTION FILE, THEY COME LAST.
# INCREASE MAX DESCRIPTION LENGTH AND MADE IT AN OPTION.
# IF head.html OR tail.html ARE FOUND IN THE TARGET DIRECTORY, THEY
# ARE USED TO GENERATE THE INDEX PAGE.
#
# MODIFIED BY ben@hartshorne.net ON 2003-06-14.  Changed the thumbnail
# name from TN_<name>.jpg to <name>_tn.jpg so that images remain in 
# alphabetical order.  If <image>_800.jpg or <image>_1024.jpg exist,
# they are considered to be alternate sizes and are not used as 
# separate images.  Multiple index.html files are created, one for
# each full size image (_800, _1024, and the real one).  If 
# <image>.txt exists, it is linked as well as supplementary info.

use File::Find;
use File::stat qw(:FIELDS);
use Getopt::Std;
use DirHandle;
use FileHandle;

# --------------------------------------------------------------------------
# start of configuration

# number of colums
$cols=3;                  # option -c

# size of thumbnails (convert -geometry, -size, -sample)
$geometry="150x150";      # option -g 120x120

# (re)size of the source images (convert -geometry, -size, -sample)
$sgeom="50%";

# quality of image : (convert -quality)
$quality=100;              # option -q 75

# max length of description
$desclength=150;   # option -d 100

# end of configuration
# --------------------------------------------------------------------------

$version = "0.002bben";

&usage(1) unless (getopts('fCdRhHrvc:q:g:s:t:', \%opts) == 1);

# just want some short help about options
&usage(0) if ($opts{"h"});

# long help
if ($opts{"H"}) {
  system("perldoc $0");
  exit 0;
}

# check if convert command exists
if (system ("convert -v > /dev/null")) {
  # command not found
  printf << "__END__";
 command convert not found

 try : rpm -qa | grep -i imageMagick to see if imageMagick is installed
 or, you can use locate convert. Make sure convert is in you path.

__END__
  exit (1);
}

if($opts{"c"}) {
  &notify ("Changed number of columns to %s\n", $opts{"c"});
  $cols=$opts{"c"};
}

if($opts{"q"}) {
  &notify ("Changed quality of thumbnails and resampling to %s\n", $opts{"q"});
  $quality=$opts{"q"};
}

if($opts{"g"}) {
  &notify ("Changed size of thumbnails (geometry) to %s\n", $opts{"g"});
  $geometry=$opts{"g"};
}

if($opts{"s"}) {
  &notify ("Changed size of source images (geometry) to %s\n", $opts{"s"});
  $sgeom=$opts{"s"};
}

if($opts{"t"}) {
  &notify ("Changed max description length to %s\n", $opts{"t"});
  $desclength=$opts{"t"};
}

# process .pthumbnailrc file.
# &do_mthumbnailrc();

unless (scalar(@ARGV)) {
  printf ("no argument, I need one or more directories.\n");
  &usage(1);
}

while ($dir = shift) {

  my @dirs = ();

  push(@dirs, $dir);

  # also include subdirs, if opt -R is present
  push(@dirs, sort(&subdirs($dir))) if($opts{"R"});

  foreach $subdir (@dirs) {

    unless (-d $subdir) {
      warn "warning : directory $subdir does not exists\n";
      next;
    }

    &notify ("Reading directory : $subdir\n");
    @files = &parsesub_dir("$subdir");

    if( scalar (@files) > 0) {
      # make an html table index
      &notify("$subdir/index.html making html index file\n");
      create_html_index($subdir, @files);
    } else {
      &notify("  warning: directory $subdir, no image found\n") unless ($opts{"R"});
    }
    &notify("\n");
  }
}


sub parsesub_dir {
  my $dir = shift;
  my @files = ();

  # don't want directories
  return unless(-d "$dir");

  my $d = new DirHandle "$dir";

  if (defined $d) {
     while (defined($file = $d->read)) { 
       next if ($file eq "." );
       next if ($file eq "..");

       # don't make thumbnails of thumbnails ...
       next if ($file =~ m/_tn\..../o);

       # skip alternate image sizes (_800 or _1024)
       next if ($file =~ m/_640\..../o);
       next if ($file =~ m/_800\..../o);
       next if ($file =~ m/_1024\..../o);

       # only gif or jpg or png
       next unless ($file =~ m/gif|jpg|png/io);

       # chech for filename caps
       $file = check_filename_caps($dir, $file) if($opts{"C"});

       # make image smaller, before making thumbnails, because of file date comparison
       image_resample($dir, $file) if($opts{"r"});

       # create a thumbnail (small image from large image)
       create_thumbnail($dir, $file);

       # return a list of files that match. Will be proced for generating of html index
       push(@files, "$dir" . "::" . "$file");
    }
  }
  return (@files);
}

sub create_thumbnail {
  my ($dir, $file) = @_;
  $file =~ /(.*)\.(...)/;
  my $thumb = sprintf "$dir/%s_tn.%s", $1, $2;
  $file = "$dir/$file";

  # make thumbnail only if image is newer than thumbnail and thumbnail file really exists
  unless (_ctime($file) > _ctime($thumb) || ! -f $thumb || $opts{"f"}) {
    &notify("  $file : thumbnail is up-to-date\n");
    return;
  }
  # system command dislike spaces in filename, so escape them
  $file =~ s# #\\ #go;
  $thumb =~ s# #\\ #go;

  # try -sample, -size, -geometry, -quality
  notify ("  $file : making thumbnail $thumb\n");
  system ("convert -quality $quality -geometry $geometry $file $thumb");

}

sub image_resample {
  my ($dir, $file) = @_;

  # use ImageMagick to make file size smaller.
  # quality can be from 0 to 100. 75 is a rather good quality.
  $file = "$dir/$file";

  # system command dislike spaces in filename, so escape them
  $file =~ s# #\\ #go;

  notify ("  $dir/$file : resampling for quality $quality\n");
  system ("convert -quality $quality -geometry $sgeom $file $file");

}

sub check_filename_caps {
  my ($dir, $file) = @_;
  my $new = $file;

  # check if filename is like this Foo.jpg, foo.JPG ...
  # and try to rename to foo.jpg

  $new =~ s/JPG/jpg/io;
  $new =~ s/GIF/gif/io;
  $new =~ s/DSCF/Dscf/io;

  unless ($new eq $file) {
    notify ("  renaming $dir/$file -> $dir/$new\n");
    rename ("$dir/$file", "$dir/$new");

    # delete TN_$file;
    $file =~ /(.*)\.(...)/;
    my $thumb = sprintf "$dir/%s_tn.%s", $1, $2;
    if ( -e $thumb) {
       unlink $thumb;
       notify ("  deleting $thumb\n")  
    }
  }
  return $new;
}


# make an html table index
sub create_html_index{
  my ($dir, @files) = @_;
  my $count = 0;
  my $fh = 'fh';
  my $d = new DirHandle "$dir";
  my %full_filelist;

  if(!defined $d){die "can't open $dir";}
  while(defined($file = $d->read)){
    $full_filelist{$file} = 1;
  }

  open ($fh, ">$dir/index.html") || die "can't create $dir/index.html";

  # read description file
  my ($descriptions, $orders) = read_description_file ("$dir/descript.ion");

  if (defined $orders) {
      @files = sort {
	  # directory and file are separated with a "::" 
	  ($crap, $aa) = split(/::/, lc($a));
	  ($crap, $bb) = split(/::/, lc($b)); 

	  return 1  unless  exists($orders->{$aa});
	  return -1 unless  exists($orders->{$bb});
	  
	  $orders->{$aa} <=> $orders->{$bb};
	  
	  } @files;
      
  } else {
      @files = sort @files;
  }

  
  # head of HTML file
  make_header($fh, $dir);

  # figure out how wide the columns should be as percentages
  # (at most two decimals)
  my $col_wid = int(10000 / $cols) / 100;
  
  foreach $file (@files) {
      # directory and file are separated with a "::" 
      ($dir, $file) = split(/::/, $file);

     # thumbnail is TN_ + $file
      $file =~ /(.*)\.(...)/;
     my $thumb = sprintf "%s_tn.%s", $1, $2;

     # if there is a description annotation for this file, get it,
     # or give a default one ONLY IF there is no description file at all.
     # ignore case (lc) in file name, because of MS-DOS, arghh
     my $description = "";
     if(exists ($descriptions->{lc($file)})) {
       $description = $descriptions->{lc($file)};
     }

     my $short_desc = $description;
     $short_desc = $file if
	 (!defined($orders) and length($short_desc) == 0);

     # if description is more than 25 char, 
     # 1/ if there is an - in desc, then cut to this char
     # 2/ else truncate to 25 char len.
     if(length($description) > $desclength) {
        if($description =~ / - /) {
          # cut desc, separator is " - ";
          ($short_desc) = $description =~ /^(.*)? - .*/o;
          # verify that string is not to long again
          $short_desc = substr($short_desc, 0, $desclength) . "..." if(length($short_desc) > $desclength);
        } else {
          $short_desc = substr($short_desc, 0, $desclength) . "...";
        }
     }

     # give some information in ALT field
     my $alt = $file;
#     $alt = $alt . " - " . _file_day($file);        # date
     $alt = $alt . " - " . _file_size("$dir/$file") ."k";  # size size in Kb
#     $alt = $alt . " - " . $description;            # description info if present

     # there must not be some " in $desc, this should cause some trouble in html code
     # should use HTML::Entities or URI::Escape, IMHO
     $alt =~ s/"//og;
     $short_desc =~ s/"//og;

      # if other sizes of this file exist, use the 800 for the link from
      # the thumbnail, and provide text links to the other sizes (1024 
      # and full)
      $file =~ m/(.*)\.(...)/;
      $six_size = "$1_640.$2";
      $eight_size = "$1_800.$2";
      $ten_size = "$1_1024.$2";
      $info = "$1.txt";
      $orig_size = $file;
      $full_html = "";
      $six_html = "";
      $eight_html = "";
      $ten_html = "";
      $info_html = "";
      if(exists($full_filelist{$ten_size})){
	$full_html = qq{<A HREF="$orig_size" style="text-decoration:none">(orig)</a>};
	$file="$ten_size";
	$ten_html = qq{<A HREF="$ten_size" style="text-decoration:none">(1024)</a>};
      }
      if(exists($full_filelist{$six_size})){
	$full_html = qq{<A HREF="$orig_size" style="text-decoration:none">(orig)</a>};
	$file="$six_size";
	$six_html = qq{<A HREF="$six_size" style="text-decoration:none">(640)</a>};
      }
      if(exists($full_filelist{$eight_size})){
	$full_html = qq{<A HREF="$orig_size" style="text-decoration:none">(orig)</a>};
	$file="$eight_size";
	$eight_html = qq{<A HREF="$eight_size" style="text-decoration:none">(800)</a>};
      }
      if(exists($full_filelist{$info})){
	$info_html = qq{<A HREF="$info" style="text-decoration:none">(info)</a>};
      }

     # now print out HTML with "table" format, and nice attributes.
     print $fh (<< "__END__");
 <TD ALIGN=CENTER VALIGN=BOTTOM WIDTH="$col_wid"><FONT face="Arial,Helvetica"><A HREF="$file" style="text-decoration:none"><IMG SRC="$thumb" ALT="$alt"><BR>$short_desc</A><BR>$six_html $eight_html $ten_html $full_html $info_html</FONT></TD>
__END__

     # make a table with $cols.
     $count++;
     if($count % $cols == 0) {
        print $fh "</tr>\n\n<tr>\n";
     }
  }

  print $fh "</tr>\n\n<tr>\n" unless ($count % $cols == 0);

  # end of HTML file
  make_tailer($fh, $dir);
}



# HTML beginnig part
sub make_header {
    my($fh, $dir) = @_;
    my $head;
    {
	$head = << "_END_";
<!doctype html public "-//w3c//dtd html 4.0 transitional//en">
<html>
<head>
   <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
   <!-- Created with MagicThumbnail, http://quinton.free.fr/MagicThumbnail/ -->
   <title>$title</title>
</head>
<body text="#FFFFFF" bgcolor="#000000" link="#FFFFFF" vlink="#0000FF" alink="#FFFF00">
&nbsp;
  
_END_
}
    if (-e "$dir/head.html") {
	my $head_fh = 'head_fh';
	if (open $head_fh, "$dir/head.html") {
	    local $/ = undef; # slurp whole file
	    $head = <$head_fh>;
	}
    }
    print $fh $head;
    print $fh "<table cellpadding=\"15\" width=\"100%\">\n<tr>\n";
}

# HTML end part
sub make_tailer {
    my ($fh, $dir) = @_;
    my $tail;
    {
	$tail = << "_END_";
     
<p>
    
<a href="../"><font face="Arial,Helvetica">Back to where you came from!</font></a>
     
</body>
</html>
_END_
}
    if (-e "$dir/tail.html") {
	my $tail_fh = 'tail_fh';
	if (open $tail_fh, "$dir/tail.html") {
	    local $/ = undef; # slurp whole file
	    $tail = <$tail_fh>;
	}
    }
    print $fh "</table>\n";
    print $fh $tail;
    
    
}

# file modification time (see fstat, stat, File::stat)
sub _mtime {
  my($file) = shift;
  stat($file);
  return $st_mtime;
}


# file creation time (see fstat, stat, File::stat)
sub _ctime {
  my($file) = shift;
  stat($file);
  return $st_ctime;
}

# file size in kb
sub _file_size {
  my($file) = shift;
  stat($file);
  return sprintf("%d", int($st_size/1024));
}


# return a string representating date of file.
sub _file_day {
  my($file) = shift;
  stat($file);

  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($st_ctime);
  
  return sprintf ("%d/%d/%d", $mday,$mon+1,$year+1900)
}

sub usage {
  my $retval = shift;
  print << "__END__";

usage : $0 -h -H -v -r -d -C -R -f -c num -q num -g geo -s geo [<dir>]+
 -h : this help
 -H : long help. like perldoc mthumbnail
 -v : verbose mode
 -r : resample image file to make them smaller
 -d : debug information
 -C : check filenames, filenames cap and convert if needed, see code source
 -R : recurse subdir
 -f : force generation of thumbnails
 -c number : number of columns, default [$cols]
 -q number : quality of resampling images, default [$quality]
 -g geometry : max size of thumbnail in pixel, default [$geometry]
 -s geometry : max (re)size of source image, default [$sgeom]
 -t size : maximum length of description, default [$desclength]
 Options to be done in future :
 -i index.html : index page filename

 See also: man mthumbnail, perldoc mthumbnail

__END__
  exit($retval);
}

sub notify {
  printf @_ if ($opts{"v"});
  return 1;
}

sub debug {
  printf @_ if ($opts{"d"});
  return 1;
}


sub subdirs {
  my($dir) = shift;
  my(@dirs) = ();
  my($file);

  my $d = new DirHandle "$dir";

  if (defined $d) {
     while (defined($file = $d->read)) { 
       next if ($file eq "." );
       next if ($file eq "..");
       if( -d "$dir/$file") {
         push (@dirs, "$dir/$file");
         push (@dirs, &subdirs("$dir/$file"));
       }
     }
  }
  undef $d;
  return (@dirs);
}


sub read_description_file {
  my ($file) = @_;

  # this is the list of description files,
  my $description = {};
  my $orders = {};
  my $count = 0;
  
  return ($description) unless (-e $file);

  my $fh = new FileHandle "$file", "r";

  return ($description) unless (defined $fh);

  while (<$fh>) {
    s/\r\n//o;  # file may be dos file \r\n
    s/\n//o;    # or may be unix compliant ...
    ($file, $desc) = m/^(\S+)?\s+(.*)$/;

    # ignore case in file name, because of MS-DOS, arghh
    $file = lc($file);

    $description->{$file} = $desc;
    $orders->{$file} = $count;
    # printf ("%s : %s\n", $file, $description->{$file});

    $count++;
  }
  undef $fh;
  return $description, $orders;
}

# process .mthumbnailrc file
sub do_mthumbnailrc {
    my $rc = $ENV{HOME} . "/.mthumbnailrc";
    if (-e $rc && -r $rc) {
	unless (do $rc) {
	    warn "$0: wrong syntax in $rc: $@\n";
	}
    }
}

__END__

=head1 NAME

mthumbnail - B<MagickThumbnail> generate thumbnails and html index page from directories of images.

=head1 SYNOPSIS

mthumbnail [options] [directory]+

I<m>thumbnail stands for I<M>agick Thumbnail. This name is because of the use
of ImageI<M>agick/convert command.

=head1 DESCRIPTION

B<MagickThumbnail> is a perl script that let you organize a directory with some images in gif,
jpg, png or any format, to publish under some place in the Internet.

It is really easy for anyone to modify this script to allow some user preferences, because
Perl scripts are pretty much readable.

It is really small, because of the use of Perl, efficient enought to parse Mo of image
files, because it rely on ImageMagick/convert command. Versatil, because it can easily
understand up to 68 images formats and as nice features. It really let you make nice,
soft and light Web gallery into a few instant. Just one command to make the gallery. After
this think about using C<sitecopy> to post it to the Internet, if needed.

B<MagickThumbnail> will do the following for you :

=over 4

=item B<HTML index page.>

Generation of HTML page, with thumnail. They are organized in rows, in HTML table. You can
choose number of colums. Defaults options for generated HTML is in perl script. So change
it in the rigth place if needed.  If the directory you are creating the HTML page in has files named "head.html" or "tail.html" then those files are used as hte header & footer of the HTML page. 

=item B<Thumbnails>

B<MagickThumbnail> easily generate thumbnail of any kind of image. You can specify thumbnail
size with C<geometry> (C<-g>) option. It can read 68 major image formats. For now, just B<gif>, B<jpg> and B<png>
formats are managed.

B<MagickThumbnail> will generate thumbnail only when it is necessary, it just compare file
modification times. Thumbnails filenames end with C<_tn>. MagickThumbnail will not
generate thumbnails of thumbnails.

=item B<Resample>

You can make image size smaller for faster access on the Internet. This is done with
B<resample> option of MagickThumbnail.

=item B<Description files>

Some programs like ACDSee (Windows) can make description of images in C<descript.ion> files.
MagickThumbnail use those descriptions files to make HTML index file more informative.
Use the HTML C<ALT> tag to describe the thumbnail.

There is two descriptions :  1/ HTML C<ALT> tag, 2/ a short description in the bottom of
the thumbnail.

In the bottom of thumbnail, if there is no description for a file, MagickThumbnail will
default to filename. It will truncate description if to long. If description contain a field
separator ' - ', it will also truncate before this special char.

HTML C<ALT> tag is a good way to give information. For now, you can see
in this order filename, file date, file size, and description if it exists.

=item B<Recurse Directories>

You can give MagickThumbnail one or more directories, and it can recurse for any subdirectory.

=item B<Change caps of filenames>

MS-DOS filename may have strange use of letters. Under Unix/Linux Web server,
you will have problems, if letter case is mixed. With MagickThumbnail, you can
bypass this problem by changing filename to nice values, use C<-C> option.

=back

=head1 OPTIONS

=over

=item -h

This will display the help screen.

=item -v

Turn on verbose mode, tried to display information as nicely as possible.

=item -R

Will recurse for subdirectories, and make an HTML index page (C<index.html>) for
each subdirectories.

=item -r

Resample image file, to reduce size/resolution. See C<-q> option, for quality; and C<-s> option for size.

=item -d

This is debug option. May be usefull for time developpement. Will normaly
not be of any effect in normal use.

=item -C

Try to change filename letter caps. Usefull when letter cap a mixed, because
of MSDOS.

=item -f

Force generation of thumbnails. Don't check file modification acces time.
Usefull when you want to change the size of thumbnails with C<-g> option.

=item -c columns

Change number of columns. Defaut is 5.

=item -q number

Quality used for resample option and for thumbnail generation. See C<convert> command.
It just look like a percentage. 75 seem to be a good ratio for quality and size.
You can play with it. This option will apply for JPG, (PNG) not for GIF files.

=item -g geometry

Size of thumbnails in pixel. Default to 120x120. Image will enter in this max rectangle.
Thumbnail will not be distorted.

=item -s geometry

Resize the source images in pixel (or as a percentage).  won't be done without C<-r> option.

=item -t size

Limit the description to C<size>, defaults to [$desclength].

=back

=head1 EXAMPLES

Some very simple examples.

=over

=item C<$ thumbnail mydir>

Will generate I<index.html> in directory I<mydir> for every image found in this directory.
Will not recurse for subdirectories. All defaults options apply.

=item C<$ thumbnail -v .>

just add verbose mode but for the current directory.

=item C<$ thumbnail -v dir1 dir2>

you can do with multiple directories at the same time.

=back

Every options are pretty to understand, so just play with them. I don't need to tell you more.

=head1 DEPENDS ON

MagickThumbnail depends on ImageMagick/convert command. So you will need to install this command
if you don't have it on you system. See C<rpm -qa rpm -qa | grep -i ImageMagick> on Linux systems. 

MagickThumbnail work with perl 5.005, convert 4.2.9, under any Unix systems.
For Windows place, see if you can find ImageMagick. There is 

=head1 KEYWORDS

perl thumbnails convert resample ImageMagick

=head1 SEE ALSO

=over 4

=item *

The official ImageMagick WWW page is now http://www.imagemagick.org/

=item *

Read the manual of C<convert> command. May be command C<identify> give you some
information about image formats.

=item *

See PerlMagick, the perl programming interface for ImageMagic. http://www.imagemagick.org/www/perl.html

=item *

Look at C<sitecopy> to post only image changes to the Web.

=back

=head1 TODO

Here are some idea for futur of B<MagickThumbnail>

=over 4

=item *

use CCS style sheet for generated page.

=item *

display format in HTML ALT tag, and thumbnail labels.

=item *

generate a main HTML index page for instant access to all subdirectories.

=item *

user defined configuration file. should be easy with perl.

=item *

add a copy-dir option, to take care of new files from one directory to
an other, with file modification.

=item *

make documentation more clean. If you can help me with english, don't hesitate ;-)

=back

=head1 AUTHOR

Marc Quinton  / http://quinton.free.fr/MagickThumbnail/

If you really need to contact me, try to guess my email. It's really easy. It don't
want to publish my email to much, because of mail spaming.  It's my name@free.fr.

Download at : http://quinton.free.fr/MagickThumbnail/download/

Screenshot at : http://quinton.free.fr/MagickThumbnail/demo/

If you like this utility, feel free to make many links, send me some mails, postcards, money,
chocolate, and what ever you want.

Any comments are welcome.


=for html
<!-- Web counter --> 
<img src="http://perso0.free.fr/cgi-bin/wwwcount.cgi?df=quinton.MagicThumbnail.dat&dd=B&ft=4" alt="MagicThumbnail">


=cut
